<?php

namespace Drupal\lb_direct_add;

use Drupal\Core\Render\Element;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Url;

class LayoutBuilder implements TrustedCallbackInterface {

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRender'];
  }

  /**
   * #pre_render callback: Alters layout builder to use dropbuttons to add custom blocks.
   */
  public static function preRender($element) {
    $lb = &$element['layout_builder'];

    $section_storage = $element['#section_storage'];

    // @todo Is there a better way to find the delta value?
    $delta = 0;

    $contexts = \Drupal::service('context.repository')->getAvailableContexts();

    $storage_type = $section_storage->getStorageType();
    $storage_id = $section_storage->getStorageId();
    $block_manager = \Drupal::getContainer()->get('plugin.manager.block');

    foreach (Element::children($lb) as $section) {
      if (isset($lb[$section]['layout-builder__section'])) {

        $sec = $lb[$section]['layout-builder__section'];
        foreach (Element::children($lb[$section]['layout-builder__section']) as $region) {

          $reg = $lb[$section]['layout-builder__section'][$region];

          $definitions = $block_manager->getFilteredDefinitions('layout_builder', $contexts, [
            'section_storage' => $section_storage,
            'region' => $region,
            'list' => 'inline_blocks',
          ]);
          $blocks = $block_manager->getGroupedDefinitions($definitions);
          $links = [];
          $inline_blocks_category = (string) t('Inline blocks');
          if (isset($blocks[$inline_blocks_category])) {
            foreach ($blocks[$inline_blocks_category] as $block_id => $block) {
              $attributes = [
                'class' => [
                  'use-ajax',
                  // 'layout-builder__link',
                  // 'layout-builder__link--add',
                ],
                'data-dialog-type' => 'dialog',
                'data-dialog-renderer' => 'off_canvas',
              ];
              $attributes['class'][] = 'js-layout-builder-block-link';
              $link = [
                'title' => "Add {$block['admin_label']}",
                'url' => Url::fromRoute('layout_builder.add_block',
                  [
                    'section_storage_type' => $section_storage->getStorageType(),
                    'section_storage' => $section_storage->getStorageId(),
                    'delta' => $delta,
                    'region' => $region,
                    'plugin_id' => $block_id,
                  ]
                ),
                'attributes' => $attributes,
              ];

              $links[] = $link;
            }
          }

          $links['other'] = [
            'title' => t('More<span class="visually-hidden"> options</span>...'),
            'url' => Url::fromRoute('layout_builder.choose_block',
              [
                'section_storage_type' => $storage_type,
                'section_storage' => $storage_id,
                'delta' => $delta,
                'region' => $region,
              ],
              [
                'attributes' => [
                  'class' => [
                    'use-ajax',
                    // 'layout-builder__link',
                    // 'layout-builder__link--add',
                  ],
                  'data-dialog-type' => 'dialog',
                  'data-dialog-renderer' => 'off_canvas',
                ],
              ]
            ),
          ];

          $lb[$section]['layout-builder__section'][$region]['layout_builder_add_block']['button'] = [
            '#type' => 'dropbutton',
            '#links' => $links,
          ];

          unset($lb[$section]['layout-builder__section'][$region]['layout_builder_add_block']['link']);
        }

        // @todo Is there a better way to find the delta value?
        $delta++;
      }
    }

    return $element;
  }

}
